using System;
using System.Web;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Text;

using CMS.UIControls;
using CMS.ExtendedControls;
using CMS.GlobalHelper;
using CMS.MediaLibrary;
using CMS.CMSHelper;

public partial class CMSModules_MediaLibrary_Controls_DirectFileUploader_DirectMediaFileUploaderControl : DirectFileUploader
{
    #region "Variables"

    private bool? mIsSupportedBrowser = null;

    #endregion

    #region "Public properties"

    /// <summary>
    /// Uploaded file
    /// </summary>
    public override HttpPostedFile PostedFile
    {
        get
        {
            return ucFileUpload.PostedFile;
        }
    }


    /// <summary>
    /// File upload user control.
    /// </summary>
    public override CMSFileUpload FileUploadControl
    {
        get
        {
            return this.ucFileUpload;
        }
    }


    /// <summary>
    /// Indicates if supported browser.
    /// </summary>
    public bool IsSupportedBrowser
    {
        get
        {
            if (mIsSupportedBrowser == null)
            {
                string browser = DataHelper.GetNotEmpty(CMSContext.GetBrowserClass(), "").ToLower();
                mIsSupportedBrowser = browser.Contains("ie") || browser.Contains("gecko");
            }
            return mIsSupportedBrowser.Value;
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            // Do nothing
            Visible = false;
        }
        else
        {
            // Prepare disabled image
            // Prepare loading image
            imgLoading.ImageUrl = ResolveUrl(LoadingImageUrl);
            lblLoading.CssClass = InnerLoadingDivClass;
            lblLoading.Text = InnerLoadingDivHtml;
            pnlLoading.Style.Add("display", "none");
            bool isRTL = (IsLiveSite && CultureHelper.IsPreferredCultureRTL()) || (!IsLiveSite && CultureHelper.IsUICultureRTL());
            if (isRTL)
            {
                pnlLoading.Style.Add("text-align", "right");
            }

            // Hide uploader if browser is unsupported
            if (!IsSupportedBrowser)
            {
                plcUploader.Visible = false;
                plcStandard.Visible = true;
                imgStandard.ImageUrl = ResolveUrl(ImageUrl);
                lblStandard.Text = InnerDivHtml;
                lblStandard.CssClass = InnerDivClass;

                // Register dialog script
                Page.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);
                string modalDialog = "modalDialog('" + ResolveUrl(GetDialogUrl()) + "', 'imageEditorDialog', 300, 180);";
                imgStandard.Attributes.Add("onclick", modalDialog);
                lblStandard.Attributes.Add("onclick", modalDialog);
            }
            else
            {
                ltlCss.Text = CreateCss();

                // Initialize uploader
                HtmlGenericControl lblContainer = (HtmlGenericControl)FindControl("container");
                lblContainer.Attributes.Add("class", "container");
                this.FileUploadControl.Attributes.Add("class", "fileUpload");
                this.FileUploadControl.Attributes.Add("onchange", "document.getElementById('" + pnlLoading.ClientID + "').style.display='';document.getElementById('" + lblContainer.ClientID + "').style.display='none';" + Page.ClientScript.GetPostBackEventReference(btnHidden, null, false));

                ltlScript.Text += ScriptHelper.GetScript("SI.Files.stylizeById('" + this.FileUploadControl.ClientID + "');");

                string rtl = (isRTL) ? "var rtl = true;" : "var rtl = false;";
                ltlInnerDiv.Text = ScriptHelper.GetScript("var innerDivHtml = '" + InnerDivHtml.Replace("'", "\\\'") + "';\nvar innerDivClass='" + InnerDivClass.Replace("'", "\\\'") + "';" + rtl);
                btnHidden.Attributes.Add("style", "display:none;");
            }
        }
    }


    protected void btnHidden_Click(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            // Do nothing
        }
        else
        {
            switch (this.SourceType)
            {
                case MediaSourceEnum.MediaLibraries:
                    HandleLibrariesUpload();
                    break;

                default:
                    break;
            }
        }
    }

    #endregion


    #region "Media libraries"

    /// <summary>
    /// Provides operations necessary to create and store new library file.
    /// </summary>
    private void HandleLibrariesUpload()
    {
        MediaFileInfo mediaFile = null;

        string message = string.Empty;

        // Get related library info
        MediaLibraryInfo mli = MediaLibraryInfoProvider.GetMediaLibraryInfo(this.LibraryID);
        if (mli != null)
        {
            try
            {
                #region "Check permissions"

                if (!MediaLibraryInfoProvider.IsUserAuthorizedPerLibrary(mli, "FileCreate"))
                {
                    throw new Exception(ResHelper.GetString("media.security.nofilecreate"));
                }

                #endregion

                // No file for upload specified
                if (!ucFileUpload.HasFile)
                {
                    throw new Exception(ResHelper.GetString("media.newfile.errorempty"));
                }

                // Check the allowed extensions
                CheckAllowedExtensions();

                // Create new media file record
                mediaFile = new MediaFileInfo(ucFileUpload.PostedFile, this.LibraryID, this.LibraryFolderPath, this.ResizeToWidth, this.ResizeToHeight, this.ResizeToMaxSideSize, mli.LibrarySiteID);

                mediaFile.FileDescription = "";

                // Save the new file info
                MediaFileInfoProvider.SetMediaFileInfo(mediaFile);
            }
            catch (Exception ex)
            {
                // Creation of new media file failed
                message = ex.Message;
            }
            finally
            {
                // Create media file info string
                string mediaInfo = "";
                if ((mediaFile != null) && (mediaFile.FileID > 0) && (this.IncludeNewItemInfo))
                {
                    mediaInfo = mediaFile.FileID.ToString() + ";" + this.LibraryFolderPath.Replace('\\', '|');
                }

                // Ensure message text
                message = HTMLHelper.EnsureLineEnding(message, " ");

                // Call function to refresh parent window                                                     
                ltlScript.Text += ScriptHelper.GetScript("if ((window.parent != null) && (window.parent.InitRefresh_" + ParentElemID + " != null)){window.parent.InitRefresh_" + ParentElemID + "(" + ScriptHelper.GetString(message.Trim()) + ", false" + ((mediaInfo != "") ? ", '" + mediaInfo + "'" : "") + (InsertMode ? ", 'insert'" : ", 'update'") + ");}");
            }
        }
    }

    #endregion
}
