using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.SiteProvider;
using CMS.GlobalHelper;
using CMS.PortalEngine;
using CMS.UIControls;

public partial class CMSSiteManager_Development_PageTemplates_PageTemplate_Tree : SiteManagerPage, IPostBackEventHandler
{
    #region "Variables"

    protected bool nodeIsSelected = false;
    protected int pageTemplateId = 0;
    protected int categoryId = 0;
    protected bool expand = false;
    private string imagesUrl = "";
    private string templateName = null;

    #endregion


    #region "Request properties"

    /// <summary>
    /// Requested action
    /// </summary>
    protected string Action
    {
        get
        {
            return ValidationHelper.GetString(Request.Form["hdnAction"], "");
        }
    }


    /// <summary>
    /// Action parameter 1
    /// </summary>
    protected string Param1
    {
        get
        {
            return ValidationHelper.GetString(Request.Form["hdnParam1"], "");
        }
    }


    /// <summary>
    /// Action parameter 2
    /// </summary>
    protected string Param2
    {
        get
        {
            return ValidationHelper.GetString(Request.Form["hdnParam2"], "");
        }
    }

    #endregion

    
    protected void Page_Load(object sender, EventArgs e)
    {
        this.RegisterExportScript();
        
        imagesUrl = UrlHelper.ImagesDirectory + "/CMSSiteManager/Development/PageTemplates/";

        this.imgNewCategory.ImageUrl = imagesUrl + "addcategory.gif";
        this.imgNewTemplate.ImageUrl = imagesUrl + "addpagetemplate.gif";
        this.imgDeleteItem.ImageUrl = imagesUrl + "delete.gif";
        this.imgExportObject.ImageUrl = imagesUrl + "exportobject.gif";

        // If template was edited select this template
        pageTemplateId = QueryHelper.GetInteger("templateid", 0);
 
        // Category
        categoryId = QueryHelper.GetInteger("categoryid", 0);

        // Parent category
        categoryId = QueryHelper.GetInteger("parentcategoryid", categoryId);

        if (categoryId > 0)
        {
            expand = true;
        }

        lnkDeleteItem.Text = "<span style=\" \" class=\" \" onclick=\"if ((selectedCategoryId > 0)||(selectedTemplateId > 0)) { if (ConfirmDelete()) DeleteItem(); } \">" + ResHelper.GetString("Development-PageTemplates_Tree.DeleteSelectedItem") + "</span>";
        lnkNewCategory.Text = "<span style=\" \" class=\" \" onclick=\"ShowNewCategoryPage(); \">" + ResHelper.GetString("Development-PageTemplates_Tree.NewCategory") + "</span>";
        lnkNewTemplate.Text = "<span style=\" \" class=\" \" onclick=\"ShowNewTemplatePage(); \">" + ResHelper.GetString("Development-PageTemplates_Tree.NewTemplate") + "</span>";
        lnkExportObject.Text = "<span style=\" \" class=\" \" onclick=\"ExportObject(); \">" + ResHelper.GetString("Development-PageTemplates_Tree.ExportObject") + "</span>";
        
        TreeNode rootNode = new TreeNode();
        rootNode.Text = "<span class=\"ContentTreeItem\" onclick=\"SelectNode('root', 0, '', 1, this); \"><span class=\"Name\">" + ResHelper.GetString("Development-PageTemplates_Tree.Root") + "</span></span>";
        rootNode.Expanded = true;
        rootNode.NavigateUrl = "#";
        rootNode.Value = "0";

        this.treeTemplates.LineImagesFolder = UrlHelper.ImagesDirectory + "/Tree";
        if (CultureHelper.IsUICultureRTL())
        {
            this.treeTemplates.LineImagesFolder += "/RTL";
        }
        this.treeTemplates.ImageSet = TreeViewImageSet.Custom;

        // Fill in the tree
        this.treeTemplates.Nodes.Clear();
        treeTemplates.Nodes.Add(rootNode);
        TreeNode selectedNode = CreateSubTree(rootNode);
        treeTemplates.CollapseAll();
        if (selectedNode != null)
        {
            //treeTemplates.Nodes[treeTemplates.Nodes.IndexOf(selectedNode)].Expand();
            //selectedNode.Expand();
            ExpandFromRootToSelectedNode(selectedNode);
            //selectedNode.Parent.Parent.Expand();
            //selectedNode.Parent.Expand();
            hdnTemplateName.Value = templateName;
        }
        else
        {
            treeTemplates.Nodes[0].Expand();
        }

        //in no node is selected, select the first category
        if (!nodeIsSelected && (treeTemplates.Nodes[0].ChildNodes.Count > 0))
        {
            treeTemplates.Nodes[0].ChildNodes[0].Text = treeTemplates.Nodes[0].ChildNodes[0].Text.Replace("class=\"ContentTreeItem\"", "class=\"ContentTreeSelectedItem\" id=\"treeSelectedNode\"");
            ltlScript.Text = "<script type=\"text/javascript\">";
            ltlScript.Text += "SelectCategory(" + treeTemplates.Nodes[0].ChildNodes[0].Value + ");";
            ltlScript.Text += "</script>";
            //treeTemplates.Nodes[0].ChildNodes[0].Expand();
        }
        else if (!nodeIsSelected && (treeTemplates.Nodes[0].ChildNodes.Count == 0))
        {
            ltlScript.Text = "<script type=\"text/javascript\">";
            ltlScript.Text += "parent.frames['pt_edit'].location.href = 'PageTemplate_Category.aspx';";
            ltlScript.Text += "</script>";
        }
        else if (nodeIsSelected && (pageTemplateId == 0) && (categoryId > 0))
        {
            ltlScript.Text = "<script type=\"text/javascript\">";
            ltlScript.Text += "SelectCategory(" + categoryId + ");";
            ltlScript.Text += "</script>";
        }

        treeTemplates.ExpandImageToolTip = ResHelper.GetString("general.expand");
        treeTemplates.CollapseImageToolTip = ResHelper.GetString("general.collapse");

        ltlScript.Text += @"<input type=""hidden"" id=""confirmationText"" value=""" + ResHelper.GetString("Administration-PageTemplate_Tree.DeleteItem") + @""">";

        string script = "function ProcessRequest(action, param1, param2){ document.getElementById('hdnAction').value = action; document.getElementById('hdnParam1').value = param1; document.getElementById('hdnParam2').value = param2; " + this.ClientScript.GetPostBackEventReference(this, null) + "}\n";

        ltlScript.Text += ScriptHelper.GetScript(script);
    }


    /// <summary>
    /// Recursively creates the subtree for given node
    /// </summary>
    /// <param name="parentNode">Root node of subtree to create</param>
    /// <returns>Return selected node or null</returns>
    protected TreeNode CreateSubTree(TreeNode parentNode)
    {
        TreeNode selectedNode = null;
        int parentCategoryId = Convert.ToInt32(parentNode.Value);
        
        //create category nodes and its subtrees
        DataSet categoryDescendants = PageTemplateCategoryInfoProvider.GetDescendantCategories(parentCategoryId);
        if ((categoryDescendants != null) && (categoryDescendants.Tables.Count > 0))
        {
            foreach (DataRow descendant in categoryDescendants.Tables[0].Rows)
            {
                TreeNode newNode = new TreeNode();
                string categoryDisplayName = HTMLHelper.HTMLEncode(ValidationHelper.GetString(descendant["CategoryDisplayName"], ""));

                if (categoryId == Convert.ToInt32(descendant["CategoryID"]))
                {
                    newNode.Text = "<span style=\"\" class=\"ContentTreeSelectedItem\" id=\"treeSelectedNode\" onclick=\"SelectNode('category'," + descendant["CategoryID"].ToString() + ",'', 1, this); \"><span class=\"Name\">" + categoryDisplayName + "</span></span>";
                    ltlScript.Text = "<script type=\"text/javascript\">";
                    ltlScript.Text += "SetLinksForCategory(" + categoryId + ");";
                    ltlScript.Text += "</script>";
                    nodeIsSelected = true;
                    selectedNode = newNode;
                }
                else
                {
                    newNode.Text = "<span style=\"\" class=\"ContentTreeItem\" onclick=\"SelectNode('category'," + descendant["CategoryID"].ToString() + ",'', 1, this); \"><span class=\"Name\">" + categoryDisplayName + "</span></span>";
                }
                newNode.NavigateUrl = "#";
                newNode.Value = descendant["CategoryID"].ToString();
                newNode.ImageUrl = imagesUrl + "category16.gif";
                parentNode.ChildNodes.Add(newNode);
                TreeNode tmpNode = CreateSubTree(newNode);
                if (tmpNode != null)
                {
                	selectedNode = tmpNode;
                }
            }
        }

        //create template nodes
        DataSet templatesDescendants = PageTemplateInfoProvider.GetTemplatesInCategory(parentCategoryId);
        if ((categoryDescendants != null) && (categoryDescendants.Tables.Count > 0))
        {
            foreach (DataRow descendant in templatesDescendants.Tables[0].Rows)
            {
                bool isReusable = ValidationHelper.GetBoolean(descendant["PageTemplateIsReusable"], true);
                string reusable = isReusable ? "1" : "0";
                int currentId = Convert.ToInt32(descendant["PageTemplateID"]);
                string currentName = Convert.ToString(descendant["PageTemplateCodeName"]);
                TreeNode newNode = new TreeNode();
                if (pageTemplateId == currentId)
                {
                    newNode.Text = "<span class=\"ContentTreeSelectedItem\" id=\"treeSelectedNode\" onclick=\"SelectNode('template'," + descendant["PageTemplateID"].ToString() + "," + ScriptHelper.GetString(HTMLHelper.HTMLEncode(currentName)) + "," + reusable + ", this); \"><span class=\"Name\">" + HTMLHelper.HTMLEncode(Convert.ToString(descendant["PageTemplateDisplayName"])) + "</span></span>";
                    ltlScript.Text = "<script type=\"text/javascript\">";
                    ltlScript.Text += "SetLinksForTemplate(" + pageTemplateId + ");";
                    ltlScript.Text += "</script>";
                    nodeIsSelected = true;
                    selectedNode = newNode;
                    templateName = currentName;
                }
                else
                {
                    newNode.Text = "<span class=\"ContentTreeItem\" onclick=\"SelectNode('template'," + descendant["PageTemplateID"].ToString() + "," + ScriptHelper.GetString(HTMLHelper.HTMLEncode(currentName)) + "," + reusable + ", this); \"><span class=\"Name\">" + HTMLHelper.HTMLEncode(Convert.ToString(descendant["PageTemplateDisplayName"])) + "</span></span>";
                }
                newNode.NavigateUrl = "#";
                if (isReusable)
                {
                    newNode.ImageUrl = imagesUrl + "pagetemplate16.gif";
                }
                else
                {
                    newNode.ImageUrl = imagesUrl + "adhoc16.gif";
                }
                parentNode.ChildNodes.Add(newNode);
            }
        }
        return selectedNode;
    }


    /// <summary>
    /// Expand all nodes on the path from root node to selected node (including this one if 'expand' is set to true)
    /// </summary>
    /// <param name="selectedNode">Node to expand</param>
    protected void ExpandFromRootToSelectedNode(TreeNode selectedNode)
    {
        TreeNode parentNode = selectedNode;
        if (expand)
        {
        	parentNode.Expand();
        }
        if (parentNode != null)
        {
        	parentNode = parentNode.Parent;
        }
        while (parentNode != null)
        {
            parentNode.Expand();
            parentNode = parentNode.Parent;
        }
    }


    public void RaisePostBackEvent(string eventArgument)
    {
        int categoryId = ValidationHelper.GetInteger(Param1, 0);
        int templateId = ValidationHelper.GetInteger(Param2, 0);

        string action = this.Action.ToLower();

        // Process the request
        switch (action)
        {
            case "delete":
                if (categoryId > 0)
                {
                    // Recursively delete template category and all its descendants
                    PageTemplateCategoryInfoProvider.DeletePageTemplateCategory(categoryId);
                }

                int parentCategoryId = 0;
                if (templateId > 0)
                {
                    // Delete template
                    parentCategoryId = PageTemplateInfoProvider.GetPageTemplateInfo(templateId).CategoryID;
                    PageTemplateInfoProvider.DeletePageTemplate(templateId);
                }

                // Reload tree
                Response.Redirect("PageTemplate_Tree.aspx?parentcategoryid=" + parentCategoryId.ToString());
                break;
        }
    }


    /// <summary>
    /// Adds the alert message to the output request window
    /// </summary>
    /// <param name="message">Message to display</param>
    private void AddAlert(string message)
    {
        ltlScript.Text += ScriptHelper.GetAlertScript(message);
    }


    /// <summary>
    /// Adds the script to the output request window
    /// </summary>
    /// <param name="script">Script to add</param>
    public override void AddScript(string script)
    {
        ltlScript.Text += ScriptHelper.GetScript(script);
    }
}
